@@ -34,11 +34,14 @@ SAB :: <include|exclude> <source1, source2, ... sourceN>
 
 REL :: <include|exclude> <relation1, relation2, ... relationN>
 
+RELA :: <include|exclude> <rela1, rela2, ... relaN>  (optional)
+
 For example, if we wanted to use the MSH vocabulary with only 
 the RB/RN relations, the configuration file would be:
 
 SAB :: include MSH
 REL :: include RB, RN
+RELA :: include inverse_isa, isa
 
 or 
 
@@ -309,11 +312,6 @@ if(defined $opt_socket) {
 
 $umls = UMLS::Interface->new(\%option_hash); 
 die "Unable to create UMLS::Interface object.\n" if(!$umls);
-($errCode, $errString) = $umls->getError();
-die "$errString\n" if($errCode);
-
-&errorCheck($umls);
-
 
 foreach my $element (@fileArray) {
     
@@ -331,7 +329,6 @@ foreach my $element (@fileArray) {
     }
     else {
 	@c1 = $umls->getConceptList($input1); 
-	&errorCheck($umls);
 	$flag1 = "term";
     }
     if( ($input2=~/C[0-9]+/)) {
@@ -339,7 +336,6 @@ foreach my $element (@fileArray) {
     }
     else {
 	@c2 = $umls->getConceptList($input2); 
-	&errorCheck($umls);
 	$flag2 = "term";
     }
     
@@ -349,16 +345,7 @@ foreach my $element (@fileArray) {
     my $floatformat = join '', '%', '.', $precision, 'f';    
     foreach $cui1 (@c1) {
 	foreach $cui2 (@c2) {
-	    
-	    if($umls->validCui($cui1)) {
-		print STDERR "ERROR: The concept ($cui1) is not valid.\n";
-		exit;
-	    }
-	    
-	    if($umls->validCui($cui2)) {
-		print STDERR "ERROR: The concept ($cui2) is not valid.\n";
-		exit;
-	    }	
+	   
 	    if(! ($umls->exists($cui1)) ) { next; }
 	    if(! ($umls->exists($cui2)) ) { next; }
 		    
@@ -375,7 +362,6 @@ foreach my $element (@fileArray) {
 	    }
 	    else {
 		@shortestpaths = $umls->findShortestPath($cui1, $cui2);
-		&errorCheck($umls);
 	    }
 
 	    foreach my $path (@shortestpaths) {
@@ -395,7 +381,6 @@ foreach my $element (@fileArray) {
 		
 		    #  get one of the terms associated with the concept
 		    my ($t) = $umls->getTermList($concept); 
-		    &errorCheck($umls);
 		    #  print out the concept
 		    print "$concept ($t) "; 
 		    
@@ -403,7 +388,6 @@ foreach my $element (@fileArray) {
 		    #  the propagation count and IC count
 		    if(defined $opt_icpropagation) { 
 			my $value = $umls->getIC($concept);          
-			&errorCheck($umls);
 			my $ic = sprintf $floatformat, $value;
 			print "($ic) ";
 		    }     
@@ -413,7 +397,6 @@ foreach my $element (@fileArray) {
 		    if( (defined $opt_info) and ($i < $#shortestpath) ) {
 			my $second = $shortestpath[$i+1];
 			my @relations = $umls->getRelationsBetweenCuis($concept, $second);
-			&errorCheck($umls);
 			print " => @relations => ";
 		    }
 		}
@@ -430,14 +413,6 @@ foreach my $element (@fileArray) {
 	print "given the current view of the UMLS.\n\n";
     }
 }
-sub errorCheck
-{
-    my $obj = shift;
-    ($errCode, $errString) = $obj->getError();
-    print STDERR "$errString\n" if($errCode);
-    exit if($errCode > 1);
-}
-
 
 ##############################################################################
 #  function to output minimal usage notes
@@ -511,7 +486,7 @@ sub showHelp() {
 #  function to output the version number
 ##############################################################################
 sub showVersion {
-    print '$Id: findShortestPath.pl,v 1.18 2010/05/11 20:04:46 btmcinnes Exp $';
+    print '$Id: findShortestPath.pl,v 1.19 2010/05/24 17:57:16 btmcinnes Exp $';
     print "\nCopyright (c) 2008, Ted Pedersen & Bridget McInnes\n";
 }
 
