@@ -33,11 +33,14 @@ SAB :: <include|exclude> <source1, source2, ... sourceN>
 
 REL :: <include|exclude> <relation1, relation2, ... relationN>
 
+RELA :: <include|exclude> <rela1, rela2, ... relaN>  (optional)
+
 For example, if we wanted to use the MSH vocabulary with only 
 the RB/RN relations, the configuration file would be:
-
+    
 SAB :: include MSH
 REL :: include RB, RN
+RELA :: include inverse_isa, isa
 
 or 
 
@@ -213,10 +216,6 @@ if(defined $opt_socket) {
 
 $umls = UMLS::Interface->new(\%option_hash); 
 die "Unable to create UMLS::Interface object.\n" if(!$umls);
-($errCode, $errString) = $umls->getError();
-die "$errString\n" if($errCode);
-
-&errorCheck($umls);
 
 my $input = shift;
 my $term  = $input;
@@ -224,22 +223,15 @@ my $term  = $input;
 my @c = ();
 if($input=~/C[0-9]+/) {
     push @c, $input;
-    ($term) = $umls->getConceptList($input);
+    ($term) = $umls->getSabDefConcepts($input);
 }
 else {
-    @c = $umls->getConceptList($input);
+    @c = $umls->getSabDefConcepts($input);
 }
 
 my $printFlag = 0;
 
 foreach my $cui (@c) {
-    if($umls->validCui($cui)) {
-	print STDERR "ERROR: The concept ($cui) is not valid.\n";
-	exit;
-    }
-
-    #  make certain cui exists in this view
-    if($umls->exists($cui) == 0) { next; }	
 
     my @defs = ();
     
@@ -250,8 +242,6 @@ foreach my $cui (@c) {
 	@defs = $umls->getCuiDef($cui); 
     }
 
-    &errorCheck($umls);
-
     if($#defs >= 0) {
 	print "The definition(s) of $term ($cui):\n";
 	my $i = 1;
@@ -270,14 +260,6 @@ foreach my $cui (@c) {
 if(! ($printFlag) ) {
     print "There are no definitions for $input\n";
 }
-sub errorCheck
-{
-    my $obj = shift;
-    ($errCode, $errString) = $obj->getError();
-    print STDERR "$errString\n" if($errCode);
-    exit if($errCode > 1);
-}
-
 
 ##############################################################################
 #  function to output minimal usage notes
@@ -328,7 +310,7 @@ sub showHelp() {
 #  function to output the version number
 ##############################################################################
 sub showVersion {
-    print '$Id: getCuiDef.pl,v 1.9 2010/05/11 21:27:24 btmcinnes Exp $';
+    print '$Id: getCuiDef.pl,v 1.11 2010/05/24 23:05:10 btmcinnes Exp $';
     print "\nCopyright (c) 2008, Ted Pedersen & Bridget McInnes\n";
 }
 
