@@ -217,10 +217,6 @@ if(defined $opt_socket) {
 
 $umls = UMLS::Interface->new(\%option_hash); 
 die "Unable to create UMLS::Interface object.\n" if(!$umls);
-($errCode, $errString) = $umls->getError();
-die "$errString\n" if($errCode);
-
-&errorCheck($umls);
 
 my $input = shift;
 my $term  = $input;
@@ -228,32 +224,23 @@ my $term  = $input;
 my @c = ();
 if($input=~/C[0-9]+/) {
     push @c, $input;
-    ($term) = $umls->getConceptList($input);
+    ($term) = $umls->getSabDefConcepts($input);
 }
 else {
-    @c = $umls->getConceptList($input);
+    @c = $umls->getSabDefConcepts($input);
 }
 
 my $printFlag = 0;
 
 foreach my $cui (@c) {
-    if($umls->validCui($cui)) {
-	print STDERR "ERROR: The concept ($cui) is not valid.\n";
-	exit;
-    }
-
-    #  make certain cui exists in this view
-    if($umls->exists($cui) == 0) { next; }	
 
     my $rdef = $umls->getExtendedDefinition($cui); 
-
     my @defs = @{$rdef};
 
-    &errorCheck($umls);
-
     if($#defs >= 0) {
 	print "The definition(s) of $term ($cui):\n";
 	my $i = 1;
+	print "$#defs\n";
 	foreach $def (@defs) {
 	    print "  $i. $def\n"; $i++;
 	}
@@ -269,14 +256,6 @@ foreach my $cui (@c) {
 if(! ($printFlag) ) {
     print "There are no definitions for $input\n";
 }
-sub errorCheck
-{
-    my $obj = shift;
-    ($errCode, $errString) = $obj->getError();
-    print STDERR "$errString\n" if($errCode);
-    exit if($errCode > 1);
-}
-
 
 ##############################################################################
 #  function to output minimal usage notes
@@ -324,7 +303,7 @@ sub showHelp() {
 #  function to output the version number
 ##############################################################################
 sub showVersion {
-    print '$Id: getExtendedDef.pl,v 1.4 2010/05/11 20:04:46 btmcinnes Exp $';
+    print '$Id: getExtendedDef.pl,v 1.6 2010/05/24 23:05:10 btmcinnes Exp $';
     print "\nCopyright (c) 2008, Ted Pedersen & Bridget McInnes\n";
 }
 
